CrossLink
=========

This package provides the code as it was used in the generation of results for
the SIGGRAPH Asia 2015 paper

"CrossLink: Joint Understanding of Image and 3D Model Collections
 through Shape and Camera Pose Variations"

by Moos Hueting (University College London), Maks Ovsjanikov (Ecole Polytechnique)
and Niloy Mitra (University College London).

Make sure to check for updated versions on our project page at

http://geometry.cs.ucl.ac.uk/projects/2015/crosslink


Instructions
============

The codebase consists in two parts. The most important part is the C++ codebase,
which is built using a CMake+GCC toolchain.

C++
---

There are a number of required packages.

- Boost for program option parsing (http://boost.org)
- Eigen 3 for matrix manipulation (http://eigen.tuxfamily.org/)
- OpenCV 2.4 for image manipulation (http://opencv.org)
- LibANN for nearest neighbour finding (http://www.cs.umd.edu/~mount/ANN/)
- TinyXML-2 for XML IO (http://grinninglizard.com/tinyxml2/index.html)
- VLFeat for feature generation (http://vlfeat.org/)
- Caffe for deep feature generation (http://caffe.berkeleyvision.org/)
- SVMLight for classification (http://svmlight.joachims.org/)

When all packages are installed, make sure to tell CMake and the source where to find the packages.

1. Add your FindEigen3.cmake to the cmake subdirectory (can be a symlink)
2. Update the LIBANN, TINYXML, VLFEAT, CUDA and CAFFE directives in CMakeLists.txt
3. Update include/tet/config.h to point to the two important SVMLight executables
   as well as svm2weights.py, which is included with this release.

Compiling the code should now be simple using the standard CMake building process:

cmake .
make

The two output executables are `classifier` and `regressor` and speak for themselves.
Running the executables without arguments will output a help text.

XML Format
----------

Both `classifier` and `regressor` take as input an XML run file, which specifies training
examples and test examples, as well as settings for the algorithms. The two scripts
`runClassification.py` and `runViewClassification.py` generate these XML files, from which
it is easy to extrapolate how to generate your own. The output of the two main executables
are the same XML files, but now extended to have test scores for both the training examples
and the test examples.

Python scripts
--------------

There are 3 python scripts that encompass most use. The other included scripts are included
for reference, but are not necessary for most use cases.

quickstart.py             - Necessary to download the data, setup the database and
                            generate the ground truth
runClassification.py      - Runs 2D to 3D classification based on either HOG/KC or CNN features
runViewClassification.py  - Runs 3D to 2D view classification based on HOG/KC features

Common pipeline
---------------

Make sure the code is compiled and scripts are extracted. Modify the variables at the top of
`quickstart.py` to point to the right directories.
Then run `quickstart.py`. This accomplishes two things:

1. Scrapes all images from Bing and 3D Warehouse for the classes listed in the paper
   (WARNING: This will take a long time.)
2. Generates a MySQL database with ground truth information for images that match
   our stored ground truth data.

Now modify tet/data.py with the correct MySQL credentials, and tet/run.py with the path to the
`classifier` and `regressor` executables.

You can now use the scripts `runViewClassification.py` and `runClassification.py` to run actual
classifications. Running them without arguments gives the necessary options.
