#ifndef REGRESSORAPP_H
#define REGRESSORAPP_H 

#include "tet/app/appbase.h"

#include "tet/descriptors/descriptor.h"
#include "tet/encodings/encoding.h"

#include "tet/processing/dataprocessing.h"

#include "tet/containers/view.h"
#include "tet/containers/instance.h"
#include "tet/containers/class.h"
#include "tet/containers/set.h"

#include "tet/kernels/kernelfactory.h"

#include "tet/classifiers/svm.h"

#include "tet/io/xmlserializable.h"

#include "tet/eval/evalprocessor.h"
#include "tet/eval/msemetric.h"

namespace tet
{

namespace classification
{

namespace app
{

template <class _DescriptorType, class _EncoderType>
class RegressorApp : public AppBase, public tet::classification::io::XMLSerializable
{
public:
    typedef _DescriptorType DescriptorType;
    typedef _EncoderType EncoderType;

    typedef typename DescriptorType::InputType InputType;
    typedef typename DescriptorType::FeatureType FeatureType;
    typedef typename EncoderType::FeatureType EncodingType;

    typedef tet::classification::containers::View     View;
    typedef tet::classification::containers::Instance Instance;
    typedef tet::classification::containers::Class    Class;
    typedef tet::classification::containers::Set      Set;

    RegressorApp() : _useCached(false)
                   , _useCustom(false)
                   , _kernel(kernels::KernelFactory::get("linear"))
                   , _name("default")
                   , _cacheDir(".")
                   , _getSetDescriptors(_cacheDir)
                   , _getSetEncodings(_cacheDir) {}

    virtual const char * xmlName() const { return "regressorApp"; }

    void setDescriptor(DescriptorType* descriptor)
    { _descriptor = descriptor; }
    void setEncoder(EncoderType* encoder)
    { _encoder = encoder; }

    void setUseCached(bool useCached);
    void setUseCustom(bool useCustom)
    { _useCustom = useCustom; }

    void setKernel(kernels::KernelBase::Ptr kernel)
    { _kernel = kernel; _svm.setKernel(kernel); }

    const DescriptorType & getDescriptor() { return _descriptor; }
    const EncoderType & getEncoder() { return _encoder; }

    const Set & getSamples() { return _samples; }
    const Set & getTests() { return _tests; }

    void run();
    void train();
    void test();

    void printInfo() const;

    void readFile(std::string filename);
    void writeFile(std::string filename);
    tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);
    void deserialize(const tinyxml2::XMLElement*);

protected:

private:
    bool _useCached;
    bool _useCustom;

    kernels::KernelBase::Ptr _kernel;
    float _c, _eps;

    std::string _name;
    std::string _cacheDir;

    processing::GetSetDescriptorsCached<DescriptorType> _getSetDescriptors;
    processing::GetSetEncodingsCached<DescriptorType, EncoderType> _getSetEncodings;
    
    Set _samples;
    Set _tests;

    DescriptorType _descriptor;
    EncoderType _encoder;

    eval::EvalProcessor _eval;

    std::string _resultFile;

    tet::classification::classifiers::SVM _svm;

}; // class RegressorApp

} // namespace app

} // namespace classification

} // namespace tet

#include "impl/regressorapp.hpp"

#endif /* REGRESSORAPP_H */
