#ifndef RUNSETTINGS_H
#define RUNSETTINGS_H 

#include <string>
#include <fstream>

namespace tet
{

namespace classification
{

namespace app
{

class RunSettings
{
public:

    RunSettings() : _useExistingWeights(false), _useCached(false), _useCustom(false) {}

    size_t readFromFile(std::string filename);
    size_t writeToFile(std::string filename);

    void setSettingsFile(std::string settings)
    { _settingsFile = settings; }

    void setResultsFile(std::string results)
    { _resultsFile = results; }

    void setUseExistingWeights(bool weights)
    { _useExistingWeights = weights; }

    void setWeightsFile(std::string weights)
    { _weightsFile = weights; _useExistingWeights = true; }

    void setUseCustom(bool useCustom)
    { _useCustom = useCustom; }

    void setEncoding(std::string enc)
    { _encoding = enc; }

    void setDescriptor(std::string desc)
    { _descriptor = desc; }

    void setUseCached(bool useCached)
    { _useCached = useCached; }

    const std::string & getSettingsFile() const
    { return _settingsFile; }

    const std::string & getResultsFile() const
    { return _resultsFile; }

    const std::string & getWeightsFile() const
    { return _weightsFile; }

    const bool & getUseCustom() const
    { return _useCustom; }

    const bool & getUseExistingWeights() const
    { return _useExistingWeights; }

    const std::string & getEncoding() const
    { return _encoding; }

    const std::string & getDescriptor() const
    { return _descriptor; }

    bool getUseCached() const
    { return _useCached; }

protected:

private:
    std::string _settingsFile;
    std::string _resultsFile;

    bool _useExistingWeights;
    std::string _weightsFile;

    std::string _descriptor;
    std::string _encoding;

    bool _useCached;
    bool _useCustom;

}; // class RunSettings

} // namespace app

} // namespace classification

} // namespace tet

#endif /* RUNSETTINGS_H */
