#ifndef SVM_H
#define SVM_H 

#include <vector>
#include <string>
#include <fstream>
#include <iostream>

#include "boost/lexical_cast.hpp"

#include "eigen3/Eigen/Core"

#include "tet/config.h"

#include "tet/io/svmlight.h"

#include "tet/containers/set.h"

#include "tet/kernels/kernelfactory.h"

namespace tet
{

namespace classification
{

namespace classifiers
{

class SVM
{
public:
    SVM() : _svmModelFile("svmModel.txt")
          , _c(-1.f)
          , _eps(-1.f)
          , _kernel(kernels::KernelFactory::get("linear")) {}
    ~SVM() {}

    void train_classifier(const std::vector<Eigen::VectorXf*> & pos,
                          const std::vector<Eigen::VectorXf*> & neg,
                          const std::string weightsFile);

    void test_classifier(const std::vector<Eigen::VectorXf*> & test,
                         std::string resultFile);

    void test_classifier_existing(const std::vector<Eigen::VectorXf*> & test,
                                  std::string resultFile,
                                  std::string weightsFile);

    void train_regressor(const std::vector<Eigen::VectorXf*> & samples,
                         const std::vector<float> values,
                         const std::string weightsFile);

    void test_regressor(const std::vector<Eigen::VectorXf*> & test,
                        std::string resultFile);

    void test_regressor_existing(const std::vector<Eigen::VectorXf*> & test,
                                 std::string resultFile,
                                 std::string weightsFile);

    inline void setModelFile(std::string modelFile)
    { _svmModelFile = modelFile; }

    inline void setKernel(kernels::KernelBase::Ptr kernel)
    { _kernel = kernel; }

    inline void setC(float c)
    { _c = c; }

    inline void setEps(float eps)
    { _eps = eps; }

protected:

private:
    std::string _svmModelFile;
    float _c;
    float _eps;

    kernels::KernelBase::Ptr _kernel;

}; // class SVM

} // namespace classifiers

} // namespace classification

} // namespace tet

#endif /* SVM_H */
