#ifndef CLASS_H
#define CLASS_H 

#include <vector>
#include <string>

#include <boost/functional/hash.hpp>

#include "tet/io/xmlserializable.h"
#include "tet/containers/instance.h"

namespace tet
{

namespace classification
{

namespace containers
{

class Class : public tet::classification::io::XMLSerializable
{
public:
    Class() : _name("default") {}

    virtual const char * xmlName() const { return "class"; }

    const std::vector<Instance> & getInstances() const { return _instances; }
    std::vector<Instance> & getInstances() { return _instances; }

    size_t size() const;

    // serialization
    tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);
    void deserialize(const tinyxml2::XMLElement*);

protected:

private:
    std::string _name;
    std::vector<Instance> _instances;

}; // class Class

std::vector<float> getGTs(const Class & c);
std::vector<float> getScores(const Class & c);

std::size_t hash_value(const Class & c);

} // namespace containers

} // namespace classification

} // namespace tet

#endif /* CLASS_H */
