#ifndef INSTANCE_H
#define INSTANCE_H 

#include <vector>

#include <boost/functional/hash.hpp>

#include "tet/io/xmlserializable.h"
#include "tet/containers/view.h"

namespace tet
{

namespace classification
{

namespace containers
{

class Instance : public tet::classification::io::XMLSerializable
{
public:
    Instance() : _gt(-1) {}

    virtual const char * xmlName() const { return "instance"; }

    const std::vector<View> & getViews() const { return _views; }
    std::vector<View> & getViews() { return _views; }

    inline size_t size() const
    { return _views.size(); }

    // serialization
    tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);
    void deserialize(const tinyxml2::XMLElement*);

protected:

private:
    std::vector<View> _views;
    int _gt;

}; // class Instance

std::vector<float> getGTs(const Instance & inst);
std::vector<float> getScores(const Instance & inst);

std::size_t hash_value(const Instance & instance);

} // namespace containers

} // namespace classification

} // namespace tet

#endif /* INSTANCE_H */
