#ifndef VIEW_H
#define VIEW_H 

#include <boost/functional/hash.hpp>

#include "opencv2/opencv.hpp"

#include "tet/io/xmlserializable.h"

namespace tet
{

namespace classification
{

namespace containers
{

class View : public tet::classification::io::XMLSerializable
{
public:
    View() : _input(nullptr), _scoreComputed(false), _score(0.f),
             _gtSet(false), _gt(0.f), _filename(""), _md5("") {}

    virtual const char * xmlName() const { return "view"; }

    template <class MatType>
    cv::Mat* getInput() const;

    void setFilename(std::string filename)
    { _filename = filename; _computeMD5(); }
    std::string getFilename() const { return _filename; }

    void setScore(float score) { _score = score; _scoreComputed = true; }
    float getScore() const { return _score; }

    void setGT(float gt) { _gt = gt; _gtSet = true; }
    float getGT() const { return _gt; }

    void unloadInput() const
    { if (_input != nullptr) { _input->release(); delete _input; _input = nullptr; } }

    std::string getMD5() const { return _md5; }

    // serialization
    tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);
    void deserialize(const tinyxml2::XMLElement*);

protected:
    void _computeMD5();

private:
    mutable cv::Mat *_input;
    
    bool _scoreComputed;
    float _score;

    bool _gtSet;
    float _gt;

    std::string _filename;
    std::string _md5;

}; // class View

std::size_t hash_value(const View & view);

} // namespace containers

} // namespace classification

} // namespace tet

#include "impl/view.hpp"

#endif /* VIEW_H */
