#ifndef CNNDESCRIPTOR_H
#define CNNDESCRIPTOR_H 

#include "eigen3/Eigen/Core"

#include "tet/descriptors/descriptor.h"
#include "tet/io/xmlserializable.h"

#include "opencv2/opencv.hpp"

#include "caffe/caffe.hpp"
#include "caffe/util/io.hpp"
#include "caffe/blob.hpp"

namespace tet
{
    namespace classification
    {
        namespace descriptors
        {
            class CNNDescriptor;
        }
    }
}

namespace std
{

    template <> struct hash<tet::classification::descriptors::CNNDescriptor>
    {
        size_t operator()(const tet::classification::descriptors::CNNDescriptor & desc) const;
    };

} // namespace std

namespace tet
{

namespace classification
{

namespace descriptors
{

    namespace _internal
    {
        struct CNNDescriptorTraits
        {
            typedef cv::Mat InputType;
            typedef Eigen::MatrixXf FeatureType;
            typedef cv::Mat VisType;

            typedef char MatType;
        };
    }

class CNNDescriptor : public Descriptor<_internal::CNNDescriptorTraits>
{
public:
    CNNDescriptor() { _setup(); }

    FeatureType getDescriptors(const InputType &img, VisType* vis=nullptr) const;
    VisType     getVisualization(const FeatureType &features) const;

    Descriptor<_internal::CNNDescriptorTraits>* clone() const
    {
        CNNDescriptor* desc = new CNNDescriptor;

        return desc;
    }

    void setNetProto(std::string protoFile);
    bool setTrainedModel(std::string modelFile);

    // serialization
    tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);
    void deserialize(const tinyxml2::XMLElement*);

    // hashing
    friend size_t std::hash<CNNDescriptor>::operator()(const CNNDescriptor&) const;
    friend std::size_t hash_value(const CNNDescriptor &);

protected:
    void _setup();

    std::string _deployFile;
    std::string _modelFile;
    std::string _outLayer;

    caffe::Net<float> *_net;

}; // class CNNDescriptor

std::size_t hash_value(const CNNDescriptor & desc);

} // namespace descriptors

} // namespace classification

} // namespace tet

#endif /* CNNDESCRIPTOR_H */
