#ifndef DESCRIPTOR_H
#define DESCRIPTOR_H 

#include "tet/io/xmlserializable.h"

namespace tet
{

namespace classification
{

namespace descriptors
{

//! Virtual class to be inherited by all descriptors.
/*!
  This class is meant solely as a superclass to all descriptor classes.
*/

template <class _DescriptorTraits>
class Descriptor : public tet::classification::io::XMLSerializable
{
public:
    typedef typename _DescriptorTraits::InputType InputType;
    typedef typename _DescriptorTraits::FeatureType FeatureType;
    typedef typename _DescriptorTraits::VisType VisType;

    typedef typename _DescriptorTraits::MatType MatType;

    virtual const char * xmlName() const { return "descriptor"; }

    virtual FeatureType getDescriptors(const InputType &input, VisType* vis=nullptr) const = 0;

    virtual VisType     getVisualization(const FeatureType &features) const = 0;

    virtual Descriptor<_DescriptorTraits>*  clone() const = 0;

protected:

private:

}; // class Descriptor

} // namespace descriptors

} // namespace classification

} // namespace tet

#endif /* DESCRIPTOR_H */
