#ifndef HOGDESCRIPTORSPB_H
#define HOGDESCRIPTORSPB_H 

#include <vector>

#include "eigen3/Eigen/Core"

#include "tet/descriptors/descriptor.h"
#include "tet/io/xmlserializable.h"

#include "opencv2/opencv.hpp"

#include "vl/hog.h"

namespace tet
{
    namespace classification
    {
        namespace descriptors
        {
            class HOGDescriptorSPB;
        }
    }
}


namespace std
{

    template <> struct hash<tet::classification::descriptors::HOGDescriptorSPB>
    {
        size_t operator()(const tet::classification::descriptors::HOGDescriptorSPB & desc) const;
    };

} // namespace std

namespace tet
{

namespace classification
{

namespace descriptors
{

    namespace _internal
    {
        struct HOGDescriptorSPBTraits
        {
            typedef cv::Mat InputType;
            typedef std::vector<Eigen::MatrixXf> FeatureType;
            typedef cv::Mat VisType;

            typedef float MatType;
        };
    }

class HOGDescriptorSPB : public Descriptor<_internal::HOGDescriptorSPBTraits>
{
public:
    HOGDescriptorSPB() : _nx(2), _ny(2)
    {
        _vlHog = vl_hog_new(VlHogVariantDalalTriggs, 9, VL_FALSE);
    }

    Descriptor<_internal::HOGDescriptorSPBTraits>* clone() const
    {
        HOGDescriptorSPB* desc = new HOGDescriptorSPB;

        desc->_nbins = _nbins;
        desc->_cellSize = _cellSize;
        desc->_removeZeroGradient = _removeZeroGradient;

        desc->_nx = _nx;
        desc->_ny = _ny;

        return desc;
    }

    void setBins(int bins) { _nbins = bins; }
    int getBins() const { return _nbins; }

    void setCellSize(int cellSize) { _cellSize = cellSize; }
    int getCellSize() const { return _cellSize; }

    FeatureType getDescriptors(const InputType &img, VisType* vis=nullptr) const;
    VisType     getVisualization(const FeatureType &features) const;

    VlHog* getHog()
    { return _vlHog; }

    void setRemoveZeroGradient(bool removeZeroGradient)
    { _removeZeroGradient = removeZeroGradient; }

    ~HOGDescriptorSPB()
    { vl_hog_delete(_vlHog); }

    // serialization
    tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);
    void deserialize(const tinyxml2::XMLElement*);

    // hashing
    friend size_t std::hash<HOGDescriptorSPB>::operator()(const HOGDescriptorSPB&) const;

protected:
    Eigen::MatrixXf getLocalDescriptors(const InputType &img, VisType* vis=nullptr) const;

private:
    int _nbins, _cellSize;
    bool _removeZeroGradient;

    int _nx, _ny;

    VlHog* _vlHog;
}; // class HOGDescriptorSPB

} // namespace descriptors

} // namespace classification

} // namespace tet

#endif /* HOGDESCRIPTORSPB_H */
