#ifndef SIFTDESCRIPTOR_H
#define SIFTDESCRIPTOR_H 

#include <vector>

#include "eigen3/Eigen/Core"

#include "tet/descriptors/descriptor.h"
#include "tet/io/xmlserializable.h"

#include "vl/dsift.h"

#include "opencv2/opencv.hpp"

namespace tet
{

namespace classification
{

namespace descriptors
{

    namespace _internal
    {
        struct SIFTDescriptorTraits
        {
            typedef cv::Mat InputType;
            typedef Eigen::MatrixXf FeatureType;
            typedef cv::Mat VisType;

            typedef float MatType;
        };
    }

class SIFTDescriptor : public Descriptor<_internal::SIFTDescriptorTraits>
{
public:
    SIFTDescriptor() : _step(4), _binSize(8) {}

    Descriptor<_internal::SIFTDescriptorTraits>* clone() const
    {
        SIFTDescriptor* desc = new SIFTDescriptor;

        desc->_step = _step;
        desc->_binSize = _binSize;

        return desc;
    }

    FeatureType getDescriptors(const InputType &img, VisType* vis=nullptr) const;
    VisType     getVisualization(const FeatureType &features) const;
    
    // serialization
    tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);
    void deserialize(const tinyxml2::XMLElement*);

protected:

private:
    int _step;
    int _binSize;

};

} // namespace descriptors

} // namespace classification

} // namespace tet

#endif /* SIFTDESCRIPTOR_H */
