#ifndef IDENTITYENCODING_H
#define IDENTITYENCODING_H 

#include <iostream>

#include "eigen3/Eigen/Core"

#include "tet/encodings/encoding.h"
#include "tet/io/xmlserializable.h"

namespace tet
{

namespace classification
{

namespace encodings
{

    namespace _internal
    {
        struct IdentityEncodingTraits {
            typedef float Scalar;
            typedef Eigen::Matrix<Scalar, -1, -1> SampleType;
            typedef Eigen::VectorXf               FeatureType;

            struct IdentityEncodingInitStruct {

            };

            typedef IdentityEncodingInitStruct InitStruct;

            static constexpr const char* const name = "identity";
        };


    }

class IdentityEncoding : public Encoding<_internal::IdentityEncodingTraits>
{
public:
    IdentityEncoding() {}

    template <class _DescriptorType>
    void init
    (const _DescriptorType & desc, tet::classification::processing::GetSetDescriptorsBase<_DescriptorType> & descGetter) {}

    FeatureType computeFeatures(SampleType samples) const
    { return Eigen::Map<Eigen::VectorXf>(samples.data(), samples.cols() * samples.rows()); }

    tinyxml2::XMLElement* serializeSpec(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);
    void deserializeSpec(const tinyxml2::XMLElement*);

protected:

private:

}; // class IdentityEncoding

inline std::size_t hash_value(const IdentityEncoding & enc)
{
    return 0;
}

} // namespace encodings

} // namespace classification

} // namespace tet

#endif /* IDENTITYENCODING_H */
