namespace tet
{

namespace classification
{

namespace encodings
{


template <class _CRTPTraits>
tinyxml2::XMLElement* Encoding<_CRTPTraits>::serialize
(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc)
{
    tinyxml2::XMLElement *root = doc->NewElement("encoding");
    root->SetAttribute("type", _CRTPTraits::name);
    parent->LinkEndChild(root);
    
    serializeSpec(root, doc);

    if (_featureWeights.size() > 0)
    {
        tinyxml2::XMLElement *weights = doc->NewElement("featureWeights");
        root->LinkEndChild(weights);

        for (size_t i = 0; i < _featureWeights.size(); ++i)
        {
            tinyxml2::XMLElement *weight = doc->NewElement("weight");
            weight->SetAttribute("id", (int)i);
            weight->SetText(_featureWeights[i]);

            weights->LinkEndChild(weight);
        }
    }

    return root;

}

template <class _CRTPTraits>
void Encoding<_CRTPTraits>::deserialize
(const tinyxml2::XMLElement* root)
{
    deserializeSpec(root);

    if (const tinyxml2::XMLElement *weights = root->FirstChildElement("featureWeights"))
    {
        for (const tinyxml2::XMLElement *weight = weights->FirstChildElement("weight");
             weight != NULL;
             weight = weight->NextSiblingElement("weight"))
        {
            Scalar w;
            weight->QueryFloatText(&w);
            _featureWeights.push_back(w);
        }
    }

}

} // namespace encodings

} // namespace classification

} // namespace tet
