#ifndef KCENCODINGSPB_H
#define KCENCODINGSPB_H

#include "eigen3/Eigen/Core"
#include "opencv2/opencv.hpp"

#include "tet/encodings/encoding.h"
//#include "tet/util/flanntree.h"
#include "tet/util/anntree.h"
#include "tet/io/xmlserializable.h"

namespace tet
{

namespace classification
{

namespace encodings
{

    namespace _internal
    {
        struct KCEncodingSPBTraits {
            typedef float                                       Scalar;
            typedef Eigen::VectorXf                             FeatureType;
            typedef std::vector<Eigen::Matrix<Scalar, -1, -1> > SampleType;

            struct KCEncodingSPBInitStruct {
                Eigen::Matrix<Scalar, -1, -1>* samples;
            };

            typedef KCEncodingSPBInitStruct InitStruct;

            static constexpr const char* const name = "kernelcodebookspb";
        };


    }

//! Kernel Codebook Encoding (Spatially Binned) class.
/*!
  This class implements Kernel Codebook Encoding of features (soft assignments) using K-Means clustering.
  See http://www.robots.ox.ac.uk/~vgg/research/encoding_eval/
*/

class KCEncodingSPB : public Encoding<_internal::KCEncodingSPBTraits>
{
public:
    //! Constructor.
    /*!
      \param k number of bins (and thus features) in encoding
    */
    KCEncodingSPB(int k=256) : _k(k), _maxPooling(false) {}

    //! Clone.
    /*!
      Clones this histogram encoding, returning a pointer to the new one.
    */
    /*Encoding<_internal::HistEncodingTraits>* clone()
    {
        HistEncoding* enc = new HistEncoding();
        enc->_k = _k;
        enc->_usePCA = _usePCA;
        enc->_pcaDim = _pcaDim;
        enc->_pcaBasis = _pcaBasis;

        enc->_centers = _centers;
        enc->_tree = new tet::classification::util::ANNTree<Scalar>(*_tree);

        return enc;
    }*/

    //! Sets number of bins
    /*!
      Sets the number of bins for the encoding.
      \param k number of bins (and thus features) in encoding
    */
    void setK(int k) { _k = k; _computed = false; }

    int getK() const { return _k; }

    //! Initialize encoding from samples
    /*!
      Computes cluster centers for histogram encoding.
    */
    template <class _DescriptorType>
    void init
    (const _DescriptorType & desc, tet::classification::processing::GetSetDescriptorsBase<_DescriptorType> & descGetter) {}

    //! Generate histogram encoding from samples
    /*!
      Generates histogram encoding from the samples given. If clusters
      were computed before, the encoding is generated based on these clusters.
      \param samples input samples, one row per sample
      \param recompute if true, clusters are recomputed even if clusters were computed before
      \return histogram of cluster assignments
    */
    FeatureType computeFeatures(SampleType samples) const;

    //! Returns computed cluster centers
    Eigen::MatrixXf getCenters() const { return _centers; }

    //! Returns FLANN tree
    //tet::classification::util::FLANNTree<Scalar>* getTree() { return _tree; }
    tet::classification::util::ANNTree<Scalar>* getTree() { return _tree; }
    
    // serialization
    tinyxml2::XMLElement* serializeSpec(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);
    void deserializeSpec(const tinyxml2::XMLElement*);

protected:

private:
    FeatureType computeFeaturesSingle(Eigen::Matrix<Scalar, -1, -1> samples) const;

    int _k;

    bool _maxPooling;

    Eigen::MatrixXf _centers; // computed cluster centers
    //tet::classification::util::FLANNTree<Scalar> *_tree; // tree for quick neighbour lookup
    tet::classification::util::ANNTree<Scalar> *_tree; // tree for quick neighbour lookup

}; // class KCEncodingSPB

} // namespace encodings

} // namespace classification

} // namespace tet

#endif /* KCENCODINGSPB_H */
