#ifndef EIGENIO_H
#define EIGENIO_H 

#include <iostream>
#include <fstream>

#include "eigen3/Eigen/Core"

namespace Eigen
{

template <class Matrix>
void write_binary(const char* filename, const Matrix& matrix)
{
    std::ofstream out(filename, std::ios::out | std::ios::binary | std::ios::trunc);
    typename Matrix::Index rows=matrix.rows(), cols=matrix.cols();
    out.write((char*) (&rows), sizeof(typename Matrix::Index));
    out.write((char*) (&cols), sizeof(typename Matrix::Index));
    out.write((char*) matrix.data(), rows*cols*sizeof(typename Matrix::Scalar) );
    out.close();
}

template <class Matrix>
void read_binary(const char* filename, Matrix& matrix)
{
    std::ifstream in(filename, std::ios::in | std::ios::binary);
    typename Matrix::Index rows=0, cols=0;
    in.read((char*) (&rows),sizeof(typename Matrix::Index));
    in.read((char*) (&cols),sizeof(typename Matrix::Index));
    matrix.resize(rows, cols);
    in.read( (char *) matrix.data() , rows*cols*sizeof(typename Matrix::Scalar) );
    in.close();
}

template <class Matrix>
void write_ascii(const char* filename, const Matrix& matrix)
{
    std::ofstream out;
    out.open(filename);

    for (typename Matrix::Index row = 0; row < matrix.rows(); ++row)
    {
        out << matrix(row, 0);
        for (typename Matrix::Index col = 1; col < matrix.cols(); ++col)
        {
            out << ", " << matrix(row, col);
        }
        out << std::endl;
    }

    out.close();
}

} // namespace Eigen

#endif /* EIGENIO_H */
