#ifndef SAMPLELOADER_H
#define SAMPLELOADER_H 

#include <vector>
#include <string>
#include <fstream>

#include "opencv2/opencv.hpp"

namespace tet
{

namespace classification
{

namespace io
{

template <class MatType> cv::Mat* loadSample(std::string);

template<> inline
cv::Mat* loadSample<float>(std::string filename)
{
    cv::Mat tmp = cv::imread(filename);
    cv::Mat *img = new cv::Mat(); 

    tmp.convertTo(*img, CV_32FC3, 1/255.0);

    return img;
}

template<> inline
cv::Mat* loadSample<char>(std::string filename)
{
    cv::Mat tmp = cv::imread(filename);
    cv::Mat *img = new cv::Mat(); 

    tmp.convertTo(*img, CV_8UC3);

    return img;
}

template <class MatType>
inline std::vector<cv::Mat*> loadSamples(std::string filename)
{
    std::cout << "[::loadSamples] Loading samples from " << filename << "..." << std::endl;

    std::vector<cv::Mat*> samples;

    std::ifstream listFile(filename);
    std::string line;

    while(getline(listFile, line))
    {

        cv::Mat* sample = loadSample<MatType>(line);

        samples.push_back(sample);
    }

    std::cout << "[::loadSamples] Loaded " << samples.size() << " samples. Done." << std::endl;

    return samples;
}

} // namespace io

} // namespace classification

} // namespace tet

#endif /* SAMPLELOADER_H */
