#ifndef KERNEL_H
#define KERNEL_H 

#include <memory>

#include "eigen3/Eigen/Core"

#include "tet/io/xmlserializable.h"

namespace tet
{

namespace classification
{

namespace kernels
{

class KernelBase : public io::XMLSerializable
{
public:
    typedef std::shared_ptr<KernelBase> Ptr;

    virtual float operator()(Eigen::VectorXf x, Eigen::VectorXf y) const = 0;

    virtual void deserialize(const tinyxml2::XMLElement* root) = 0;
    virtual tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc) = 0;

    virtual inline const char * xmlName() const
    { return "kernel"; }

    virtual bool isLinear() const
    { return false; }

    virtual std::string toStr() const = 0;

protected:

    ~KernelBase() {}

private:

}; // class Kernel

} // namespace kernels

} // namespace classification

} // namespace tet

#endif /* KERNEL_H */
