#ifndef POLYNOMIALKERNEL_H
#define POLYNOMIALKERNEL_H 

#include <cmath>

#include "eigen3/Eigen/Core"

#include "tet/kernels/kernelbase.h"

namespace tet
{

namespace classification
{

namespace kernels
{

class PolynomialKernel : public KernelBase
{
public:
    PolynomialKernel(float a=1.0f, float r=1.0f, int d=2) : _a(a), _r(r), _d(d) {}

    virtual inline float operator()(Eigen::VectorXf x, Eigen::VectorXf y) const
    { return std::pow(_a * x.transpose() * y + _r, _d); }

    inline virtual std::string toStr() const
    { return "polynomial"; }

    inline float getA() const { return _a; }
    inline float getR() const { return _r; }
    inline int   getD() const { return _d; }

    virtual void deserialize(const tinyxml2::XMLElement* root);
    virtual tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);

protected:

private:
    float _a, _r;
    int _d;

}; // class LinearKernel

} // namespace kernels

} // namespace classification

} // namespace tet

#endif /* POLYNOMIALKERNEL_H */
