#ifndef RBFKERNEL_H
#define RBFKERNEL_H 

#include <cmath>

#include "eigen3/Eigen/Core"

#include "tet/kernels/kernelbase.h"

namespace tet
{

namespace classification
{

namespace kernels
{

class RBFKernel : public KernelBase
{
public:
    RBFKernel(float gamma=1.0f) : _gamma(gamma) {}

    virtual inline float operator()(Eigen::VectorXf x, Eigen::VectorXf y) const
    { return std::exp(-_gamma * (x - y).transpose() * (x - y)); }

    inline virtual std::string toStr() const
    { return "rbf"; }

    inline float getGamma() const { return _gamma; }

    virtual void deserialize(const tinyxml2::XMLElement* root);
    virtual tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);

protected:

private:
    float _gamma;

}; // class LinearKernel

} // namespace kernels

} // namespace classification

} // namespace tet

#endif /* RBFKERNEL_H */
