#ifndef SIGMOIDKERNEL_H
#define SIGMOIDKERNEL_H 

#include <cmath>

#include "eigen3/Eigen/Core"

#include "tet/kernels/kernelbase.h"

namespace tet
{

namespace classification
{

namespace kernels
{

class SigmoidKernel : public KernelBase
{
public:
    SigmoidKernel(float a=1.0f, float r=1.0f) : _a(a), _r(r) {}

    virtual inline float operator()(Eigen::VectorXf x, Eigen::VectorXf y) const
    { return std::tanh(_a * x.transpose() * y + _r); }

    inline virtual std::string toStr() const
    { return "sigmoid"; }

    inline float getA() const { return _a; }
    inline float getR() const { return _r; }

    virtual void deserialize(const tinyxml2::XMLElement* root);
    virtual tinyxml2::XMLElement* serialize(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc);

protected:

private:
    float _a, _r;

}; // class LinearKernel

} // namespace kernels

} // namespace classification

} // namespace tet

#endif /* SIGMOIDKERNEL_H */
