#ifndef SERIALIZATION_H
#define SERIALIZATION_H 

#include <string>
#include "eigen3/Eigen/Core"

#include "tet/io/eigenio.h"

namespace tet
{

namespace classification
{

namespace processing
{

// deserialization
template <class Type>
Type deserialize(const std::string&);

template <> inline
Eigen::MatrixXf deserialize<Eigen::MatrixXf>(const std::string & fn)
{
    Eigen::MatrixXf mat;
    Eigen::read_binary(fn.c_str(), mat);
    return mat;
}

template <> inline
Eigen::VectorXf deserialize<Eigen::VectorXf>(const std::string & fn)
{
    Eigen::VectorXf vec;
    Eigen::read_binary(fn.c_str(), vec);
    return vec;
}

// serialization
template <class Type>
void serialize(const Type &, const std::string &);

template <> inline
void serialize<Eigen::MatrixXf>(const Eigen::MatrixXf & mat, const std::string & fn)
{
    Eigen::write_binary(fn.c_str(), mat);
}

template <> inline
void serialize<Eigen::VectorXf>(const Eigen::VectorXf & vec, const std::string & fn)
{
    Eigen::write_binary(fn.c_str(), vec);
}

} // namespace processing

} // namespace classification

} // namespace tet

#endif /* SERIALIZATION_H */
