#ifndef EIGENUTILS_H
#define EIGENUTILS_H 

#include "eigen3/Eigen/Core"


template <class _MatrixType>
inline _MatrixType concatMatricesVert(const std::vector<_MatrixType*> & matrices)
{
    size_t totRows = 0;
    for (size_t i = 0; i < matrices.size(); ++i)
    {
        totRows += matrices[i].rows();
    }

    _MatrixType resMat(totRows, matrices[0].cols());

    resMat.block(0, 0, matrices[0].rows(), matrices[0].cols()) =
        matrices[0];
    for (size_t i = 1; i < matrices.size(); ++i)
    {
        resMat.block(matrices[i - 1].rows(), 0, matrices[i].rows(), matrices[i].cols());
    }

    return resMat;
}

namespace Eigen
{

inline std::size_t hash_value(const Eigen::Matrix<float, -1, -1> & mat)
{
    size_t seed = 0;
    for (int i = 0; i < mat.size(); ++i)
    {
        auto elem = *(mat.data() + i);
        boost::hash_combine(seed, elem);
    }
    return seed;
}

} // namespace Eigen

#endif /* EIGENUTILS_H */
