#include <iostream>

#include "opencv2/core/eigen.hpp"

#include "tet/util/anntree.h"

namespace tet
{

namespace classification
{

namespace util
{

template <class _Scalar>
ANNTree<_Scalar>::ANNTree(Eigen::Matrix<_Scalar, -1, -1> datapoints)
{
    std::cout << "[ANNTree::ANNTree | Building tree] Starting..." << std::endl;
    ANNpointArray annPoints = annAllocPts(datapoints.rows(), datapoints.cols());
    for (int i = 0; i < datapoints.rows(); ++i)
    {
        for (int j = 0; j < datapoints.cols(); ++j)
        {
            annPoints[i][j] = datapoints(i, j);
        }
    }

    _ann_tree = new ANNkd_tree(annPoints, datapoints.rows(), datapoints.cols());

    std::cout << "[ANNTree::ANNTree | Building tree] Done." << std::endl;
}

template <class _Scalar>
std::vector<size_t> ANNTree<_Scalar>::findNN(Eigen::Matrix<_Scalar, -1, 1> datapoint, int n) const
{
    ANNidx* neighbour = new ANNidx[n];
    ANNdist* distance = new ANNdist[n];
    ANNpoint annPoint = annAllocPt(datapoint.rows());

    for (int i = 0; i < datapoint.rows(); ++i)
    {
        annPoint[i] = datapoint(i);
    }

    //std::cout << "[ANNTree::findNN | Finding neighbour]" << std::endl;
    _ann_tree->annkSearch(annPoint, n, neighbour, distance);

    std::vector<size_t> idx(n);
    for (int i = 0; i < n; ++i)
    {
        idx[i] = neighbour[i];
    }

    delete[] neighbour;
    delete[] distance;
    annDeallocPt(annPoint);

    return idx;
}

template <class _Scalar>
size_t ANNTree<_Scalar>::findNN(Eigen::Matrix<_Scalar, -1, 1> datapoint) const
{
    ANNidx* neighbour = new ANNidx;
    ANNdist* distance = new ANNdist;
    ANNpoint annPoint = annAllocPt(datapoint.rows());

    for (int i = 0; i < datapoint.rows(); ++i)
    {
        annPoint[i] = datapoint(i);
    }

    //std::cout << "[ANNTree::findNN | Finding neighbour]" << std::endl;
    _ann_tree->annkSearch(annPoint, 1, neighbour, distance);

    size_t res = *neighbour;

    delete neighbour;
    delete distance;
    annDeallocPt(annPoint);

    return res;
}

inline Eigen::MatrixXf computeKMeans(const Eigen::MatrixXf & samples, int k)
{
    cv::Mat samplesCV, labelsCV, centersCV;

    cv::eigen2cv(samples, samplesCV);
    samplesCV.convertTo(samplesCV, CV_32F);

    kmeans(samplesCV, k, labelsCV, cv::TermCriteria(cv::TermCriteria::EPS + cv::TermCriteria::COUNT, 10, 0.001), 3, cv::KMEANS_PP_CENTERS, centersCV);

    Eigen::MatrixXf centers;
    cv2eigen(centersCV, centers);

    return centers;
}

} // namespace util

} // namespace classification

} // namespace tet
