#include <iostream>

#include "tet/util/flanntree.h"

namespace tet
{

namespace classification
{

namespace util
{

template <class _Scalar>
FLANNTree<_Scalar>::FLANNTree(Eigen::Matrix<_Scalar, -1, -1> datapoints)
{
    flann::Matrix<_Scalar> flannData(new _Scalar[datapoints.rows() * datapoints.cols()], datapoints.rows(), datapoints.cols());
    for (size_t i = 0; i < datapoints.rows(); ++i)
    {
        for (size_t j = 0; j < datapoints.cols(); ++j)
        {
            flannData[i][j] = datapoints(i, j);
        }
    }

    _index = new flann::Index<flann::L2<_Scalar> >(flannData, flann::KDTreeIndexParams(4));
    _index->buildIndex();

    delete[] flannData.ptr();

}

template <class _Scalar>
std::vector<std::vector<size_t> > FLANNTree<_Scalar>::findNN(Eigen::Matrix<_Scalar, -1, -1> datapoints, int n) const
{
    flann::Matrix<_Scalar> query(new _Scalar[datapoints.rows() * datapoints.cols()], datapoints.rows(), datapoints.cols());
    for (size_t i = 0; i < datapoints.rows(); ++i)
    {
        for (size_t j = 0; j < datapoints.cols(); ++j)
        {
            query[i][j] = datapoints(i, j);
        }
    }

    flann::Matrix<int> indices(new int[query.rows*n], query.rows, n);
    flann::Matrix<_Scalar> dists(new _Scalar[query.rows*n], query.rows, n);

    _index->knnSearch(query, indices, dists, n, flann::SearchParams(128));

    std::vector<std::vector<size_t> > idx(query.rows);
    for (size_t i = 0; i < query.rows; ++i)
    {
        std::vector<size_t> neighbours(n);
        for (size_t j = 0; j < n; ++j)
        {
            neighbours[j] = indices[i][j];
        }
        idx[i] = neighbours;
    }

    delete[] indices.ptr();
    delete[] dists.ptr();
    delete[] query.ptr();

    return idx;
}

/*template <class _Scalar>
size_t FLANNTree<_Scalar>::findNN(Eigen::Matrix<_Scalar, -1, 1> datapoint) const
{
    flann::Matrix<_Scalar> query(static_cast<_Scalar *>(datapoint.data()), 1, datapoint.rows());
    flann::Matrix<int> indices(new int, 1, 1);
    flann::Matrix<_Scalar> dists(new _Scalar, 1, 1);

    _index->knnSearch(query, indices, dists, 1, flann::SearchParams(128));

    _Scalar res = indices[0][0];

    delete[] query.ptr();
    delete[] indices.ptr();
    delete[] dists.ptr();

    return res;
}*/

} // namespace util

} // namespace classification

} // namespace tet
