#ifndef KERNELUTILS_H
#define KERNELUTILS_H 

#include "tet/kernels/kernelbase.h"
#include "tet/kernels/linearkernel.h"
#include "tet/kernels/polynomialkernel.h"
#include "tet/kernels/rbfkernel.h"
#include "tet/kernels/sigmoidkernel.h"

#define KERNEL_ARGS(KERNEL_TYPE, BODY) \
    if (auto derivedKernel = std::dynamic_pointer_cast<KERNEL_TYPE>(kernel)) \
    { \
        BODY \
    } else 
#define KERNEL_ARGS_END { return ""; }

namespace tet
{

namespace classification
{

namespace util
{

using namespace kernels;

inline std::string kernelToSVMLightArgs(KernelBase::Ptr kernel)
{
    KERNEL_ARGS(LinearKernel,
        return " -t 0 ";
    )
    KERNEL_ARGS(PolynomialKernel,
        std::stringstream args;
        args << " -t 1 -d " << derivedKernel->getD() << " -r " << derivedKernel->getR()
             << " -s " << derivedKernel->getA() << " ";
        return args.str();
    )
    KERNEL_ARGS(RBFKernel,
        std::stringstream args;
        args << " -t 2 -g " << derivedKernel->getGamma() << " ";
        return args.str();
    )
    KERNEL_ARGS(SigmoidKernel,
        std::stringstream args;
        args << " -t 3 -r " << derivedKernel->getR() << " -s " << derivedKernel->getA() << " ";
        return args.str();
    )
    KERNEL_ARGS_END
}

} // namespace util

} // namespace classification

} // namespace tet

#undef KERNEL_ARGS
#undef KERNEL_ARGS_END

#endif /* KERNELUTILS_H */
