#ifndef SAMPLINGUTILS_H
#define SAMPLINGUTILS_H 

#include <cmath>

#include "tet/descriptors/hogdescriptor.h"
#include "tet/descriptors/hogdescriptorspb.h"

#include "tet/containers/set.h"

#include "tet/processing/dataprocessing.h"

namespace tet
{

namespace classification
{

namespace util
{

template <class _DescriptorType>
typename _DescriptorType::FeatureType sampleDescriptorsFromSet
(const  tet::classification::containers::Set & set,
 size_t n,
 const _DescriptorType & desc,
 tet::classification::processing::GetSetDescriptorsBase<_DescriptorType> & descGetter)
{
    typedef typename _DescriptorType::FeatureType FeatureType;

    size_t totSize = 0;
    std::vector<FeatureType*> descriptors = descGetter(set, desc);

    for (auto it = descriptors.begin(); it != descriptors.end(); ++it)
    {
        totSize += (*it)->rows();
    }

    FeatureType samples;

    if (totSize < n)
    {
        samples = FeatureType(totSize, descriptors[0]->cols());
        int curIdx = 0;
        for (auto it = descriptors.begin(); it != descriptors.end(); ++it)
        {
            for (int i = 0; i < (*it)->rows(); ++i)
            {
                samples.row(curIdx) = (*it)->row(i);
                curIdx += 1;
            }
        }
    } else {
        samples = FeatureType(n, descriptors[0]->cols());

        const size_t offset = floor(totSize / n);
        size_t curIdx = 0;
        size_t curOffset = 0;
        for (size_t i = 0; i < n; ++i)
        {
            if (descriptors[curIdx]->rows() == 0)
            {
                curIdx += 1;
                continue;
            }
            samples.row(i) = descriptors[curIdx]->row(curOffset);
            if ((curOffset + offset) < descriptors[curIdx]->rows())
            {
                curOffset += offset;
            } else {
                curIdx += 1;
                curOffset = 0;
            }
        }
    }

    return samples;
    
}

/*template <class _EncoderType>
Eigen::MatrixXf* sampleDescriptorsFromSet
(tet::classification::containers::Set* set, size_t n)
{
    typedef typename tet::classification::descriptors::HOGDescriptor::FeatureType FeatureType;

    size_t totSize = 0;
    std::vector<FeatureType*> desc = set->getAllDescriptors();
    for (auto it = desc.begin(); it != desc.end(); ++it)
    {
        totSize += (*it)->rows();
    }

    FeatureType *samples;

    if (totSize < n)
    {
        samples = new FeatureType(totSize, desc[0]->cols());
        int curIdx = 0;
        for (auto it = desc.begin(); it != desc.end(); ++it)
        {
            for (int i = 0; i < (*it)->rows(); ++i)
            {
                samples->row(curIdx) = (*it)->row(i);
                curIdx += 1;
            }
        }
    } else {
        samples = new FeatureType(n, desc[0]->cols());

        const size_t offset = floor(totSize / n);
        size_t curIdx = 0;
        size_t curOffset = 0;
        for (size_t i = 0; i < n; ++i)
        {
            if (desc[curIdx]->rows() == 0)
            {
                curIdx += 1;
                continue;
            }
            samples->row(i) = desc[curIdx]->row(curOffset);
            if ((curOffset + offset) < desc[curIdx]->rows())
            {
                curOffset += offset;
            } else {
                curIdx += 1;
                curOffset = 0;
            }
        }
    }

    return samples;
    
}*/

/*template <class _EncoderType>
Eigen::MatrixXf* sampleDescriptorsFromSet
(tet::classification::containers::Set<tet::classification::descriptors::HOGDescriptorSPB, _EncoderType>* set, size_t n)
{
    typedef tet::classification::descriptors::HOGDescriptorSPB DescriptorType;
    typedef DescriptorType::FeatureType FeatureType;

    size_t totSize = 0;
    std::vector<FeatureType*> desc = set->getAllDescriptors();

    size_t gridSize = desc[0]->size();
    for (auto it = desc.begin(); it != desc.end(); ++it)
    {
        for (size_t i = 0; i < gridSize; ++i)
        {
            totSize += (**it)[i].rows();
        }
    }

    Eigen::MatrixXf *samples;

    if (totSize < n)
    {
        samples = new Eigen::MatrixXf(totSize, (*(desc[0]))[0].cols());
        int curIdx = 0;
        for (auto it = desc.begin(); it != desc.end(); ++it)
        {
            for (size_t i = 0; i < gridSize; ++i)
            {
                for (size_t j = 0; j < (**it)[i].rows(); ++j)
                {
                    samples->row(curIdx) = (**it)[i].row(j);
                    curIdx += 1;
                }
            }
        }
    } else {
        samples = new Eigen::MatrixXf(n, (*(desc[0]))[0].cols());

        const size_t offset = floor(totSize / n);
        size_t curIdx = 0;
        size_t curGridIdx = 0;
        size_t curOffset = 0;
        size_t sampIdx = 0;

        for (size_t i = 0; i < n; ++i)
        {
            if ((*(desc[curIdx]))[curGridIdx].rows() == 0)
            {
                if (curGridIdx + 1 == gridSize)
                {
                    curIdx += 1;
                    curGridIdx = 0;
                } else {
                    curGridIdx += 1;
                }
                continue;
            }
            samples->row(sampIdx) = (*(desc[curIdx]))[curGridIdx].row(curOffset);
            sampIdx += 1;
            if ((curOffset + offset) < (*(desc[curIdx]))[curGridIdx].rows())
            {
                curOffset += offset;
            } else {
                if (curGridIdx + 1 == gridSize)
                {
                    curIdx += 1;
                    curGridIdx = 0;
                    curOffset = 0;
                } else {
                    curGridIdx += 1;
                    curOffset = 0;
                }
            }
        }

        samples->conservativeResize(sampIdx, (*(desc[0]))[0].cols());
    }

    return samples;
}*/

} // namespace util

} // namespace classification

} // namespace tet

#endif /* SAMPLINGUTILS_H */
