'''
This is designed for the new Azure Marketplace Bing Search API (released Aug 2012)

Inspired by https://github.com/mlagace/Python-SimpleBing and 
http://social.msdn.microsoft.com/Forums/pl-PL/windowsazuretroubleshooting/thread/450293bb-fa86-46ef-be7e-9c18dfb991ad
'''

import requests # Get from https://github.com/kennethreitz/requests
import string
import json
import shutil
import os
import sys
from PIL import Image

class BingSearchAPI():
    bing_api = "https://api.datamarket.azure.com/Data.ashx/Bing/Search/v1/Composite?"
    
    def __init__(self, key):
        self.key = key

    def replace_symbols(self, request):
        # Custom urlencoder.
        # They specifically want %27 as the quotation which is a single quote '
        # We're going to map both ' and " to %27 to make it more python-esque
        request = string.replace(request, "'", '%27')
        request = string.replace(request, '"', '%27')
        request = string.replace(request, '+', '%2b')
        request = string.replace(request, ' ', '%20')
        request = string.replace(request, ':', '%3a')
        return request
        
    def search(self, sources, query, params):
        ''' This function expects a dictionary of query parameters and values.
            Sources and Query are mandatory fields. 
            Sources is required to be the first parameter.
            Both Sources and Query requires single quotes surrounding it.
            All parameters are case sensitive. Go figure.

            For the Bing Search API schema, go to http://www.bing.com/developers/
            Click on Bing Search API. Then download the Bing API Schema Guide
            (which is oddly a word document file...pretty lame for a web api doc)
        '''
        request =  'Sources="' + sources    + '"'
        request += '&Query="'  + str(query) + '"'
        for key,value in params.iteritems():
            request += '&' + key + '=' + str(value) 
        request = self.bing_api + self.replace_symbols(request)
        return requests.get(request, auth=(self.key, self.key))

def save_image(image_url, outfile):
    try:
        r = requests.get(image_url, stream=True)
        if r.status_code == 200:
            with open(outfile, 'wb') as f:
                for chunk in r.iter_content(1024):
                    f.write(chunk)
            return 1
    except:
        print "couldn't get a request"
        return 0                    

def resize_image(outfile, maxdim):
    try:
        im = Image.open(outfile)
        (w,h) = im.size
        scale = 1
        if (w>maxdim):
            scale = maxdim/float(w)
        if (h*scale > maxdim):
            scale = maxdim/float(h)
        
        im = im.resize((int(round(w*scale)),int(round(h*scale))), Image.ANTIALIAS)
        im.save(outfile, 'jpeg')
        return 1
    except:
        print "couldn't open image file"
        return 0                    

def scrape(query_string, api_key, outdir):
    bing = BingSearchAPI(api_key)
    count  = 0

    outdir = os.path.join(outdir, str.replace(query_string," ","_")) + "/"
    outsmalldir = os.path.join(outdir, 'res_400x300') + "/"

    if not os.path.exists(outdir):
        os.makedirs(outdir)
    if not os.path.exists(outsmalldir):
        os.makedirs(outsmalldir)

    good_images = []
    for skip_results in range(0, 300, 50):
        params = {'$format': 'json',
                  '$top': 50,
                  '$skip': skip_results}
        print "Searching for " + query_string
        results = bing.search('image',query_string,params).json() # requests 1.0+

        print "found " + str(len(results['d']['results'][0]['Image'])) + " results"

        for result in results['d']['results'][0]['Image']:          
            if(result['ContentType']=="image/jpeg"):
                image_url = result['MediaUrl']
                good_images.append(image_url)

    url_file = outdir + "urls.txt"            
    f = open(url_file, 'wb')
    for image_url in good_images:
        f.write(image_url.encode('utf8') + "\n")
    f.close()      
    
    for image_url in good_images:
        print "writing image " + str(count)

        outfile = outdir + "img" + str(count) + ".jpg"
        if(save_image(image_url,outfile) == 1):
            if(resize_image(outfile,1400)):
                count = count + 1
        outsmallfile = outsmalldir + "img" + str(count) + ".jpg"
        img = Image.open(outfile)
        img.thumbnail((400, 300), Image.ANTIALIAS)
        outimg = Image.new('RGB', (400, 300), (255, 255, 255))
        outimg.paste(img, ((400 - img.size[0]) / 2, (300 - img.size[1]) / 2))
        outimg.save(outsmallfile)
