import hashlib
import MySQLdb
import argparse

def hashfile(afile, hasher, blocksize=65536):
    buf = afile.read(blocksize)
    while len(buf) > 0:
        hasher.update(buf)
        buf = afile.read(blocksize)
    return hasher.hexdigest()

def restoreGT(mysqluser, mysqlpass):
    db = MySQLdb.connect(host="localhost", user=mysqluser, passwd=mysqlpass, db="tdw")
    cur = db.cursor()
    try:
        tdw_gt_file = open("tdw_classification_gt.txt", "rb")
        hashToGT = {}

        for line in tdw_gt_file.readlines():
            digestGTPair = line.split(' ')
            hashToGT[digestGTPair[0]] = digestGTPair[1]

        query = "SELECT filename, instance_id FROM tdw_views"
        cur.execute(query)
        db.commit()

        for (filename, instance_id) in cur.fetchall():
            digest = hashfile(open(filename, 'rb'), hashlib.sha256())
            gt = hashToGT.get(digest, -1)
            if gt >= 0:
                query = "UPDATE tdw_instances SET gt={0} WHERE id={1}".format(gt, instance_id)
                cur.execute(query)
                db.commit()

    except IOError:
        print("Could not open tdw_classification_gt.txt.")
        exit()

    try:
        tdw_align_file = open("tdw_align_gt.txt", "rb")
        hashToGT = {}

        for line in tdw_align_file.readlines():
            digestGTPair = line.split(' ')
            hashToGT[digestGTPair[0]] = digestGTPair[1]

        query = "SELECT filename, file_id, instance_id FROM tdw_views"
        cur.execute(query)
        db.commit()

        for (filename, file_id, instance_id) in cur.fetchall():
            digest = hashfile(open(filename, 'rb'), hashlib.sha256())
            gt = int(hashToGT.get(digest, -1))
            if gt >= 0:
                gt = (gt - file_id) % 36
                query = "UPDATE tdw_instances SET align_offset={0} WHERE id={1}".format(gt, instance_id)
                cur.execute(query)
                db.commit()
    except IOError:
        print("Could not open tdw_align_gt.txt.")
        exit()

    try:
        bing_view_file = open("bing_view_gt.txt", "rb")
        hashToGT = {}

        for line in bing_view_file.readlines():
            digestGTPair = line.split(' ')
            hashToGT[digestGTPair[0]] = digestGTPair[1]

        query = "SELECT filename FROM bing_instances"
        cur.execute(query)
        db.commit()

        for filename in cur.fetchall():
            digest = hashfile(open(filename[0], 'rb'), hashlib.sha256())
            gt = hashToGT.get(digest, -1)
            if gt >= 0:
                query = "UPDATE bing_instances SET gtView={0} WHERE filename='{1};'".format(filename, gt)
                cur.execute(query)
                db.commit()
    except IOError:
        print("Could not open bing_view_gt.txt.")
        exit()

if __name__=="__main__":
    restoreGT()
