'''
This is designed for scraping 3D Warehouse (July 2015)

Inspired by https://github.com/mlagace/Python-SimpleBing and 
http://social.msdn.microsoft.com/Forums/pl-PL/windowsazuretroubleshooting/thread/450293bb-fa86-46ef-be7e-9c18dfb991ad
'''
from PIL import Image
import sys
import requests
import string
import json
import shutil
import os
import urllib
import urllib2
from StringIO import StringIO

def save_image(image_url, outfile):
    try:
        response = requests.get(image_url)
        img = Image.open(StringIO(response.content))
        img.save(outfile, 'jpeg')
        return 1
    except:
        e = sys.exc_info()[0]
        print "failed to open image " + outfile
        print "error: " + str(e)
        return 0                    


def crop_smontage(file):
    try:
        im = Image.open(str(file))
        outdir = str(file[:-4]) + "_views/"
        if not os.path.exists(outdir):
            os.mkdir(outdir)
        bname = os.path.basename(file)
        (w,h) = im.size
        ws = w/36
        for k in range(0, 36):
            box = (k*ws, 0, k*ws+ws, h)
            outfile = file[:-4]+"_view_"+str(k)+".jpg"
            im.crop(box).save(outdir + bname[:-4] + "_view_"+"%02d"%(k)+".jpg","jpeg")
        return 1
    except:
        e = sys.exc_info()[0]
        print "crop swivel error " + file
        print str(e)
        return 0                    

def scrape(query_string, outdir):
    count  = 0

    outdir = os.path.join(outdir, str.replace(query_string," ","_")) + "/"

    if not os.path.exists(outdir):
        os.makedirs(outdir)
        os.makedirs(os.path.join(outdir, "lt"))
        os.makedirs(os.path.join(outdir, "smontage"))
        os.makedirs(os.path.join(outdir, "bot_lt"))
        os.makedirs(os.path.join(outdir, "bot_smontage"))
        os.makedirs(os.path.join(outdir, "entities"))

    good_images = []
    model_file = outdir + "models.txt"
    fm = open(model_file, 'w')
    fm.close()
    
    for skip_results in range(0, 300, 50):
        print "Searching for " + query_string
        starti = skip_results + 1
        endi = skip_results + 50
        address = "https://3dwarehouse.sketchup.com/3dw/Search?q="+urllib.quote(query_string)+"&startRow="+str(starti)+"&endRow="+str(endi)
        response = urllib2.urlopen(address)
        data = json.load(response)

        print "found " + str(len(data['entries'])) + " results"
        model_file = outdir + "models.txt"
        fm = open(model_file, 'a')   
        for entry in data['entries']: 
            id = entry['id']
            fm.write(str(id)+"\n")
        fm.close()

        for entry in data['entries']:

            id = entry['id']
            entity = json.load(urllib2.urlopen("https://3dwarehouse.sketchup.com/3dw/getEntity?id="+id))

            outfile = outdir + "entities/" + "img" + "%04d"%(count) + ".txt"
            fk = open(outfile,'w')
            fk.write(str(json.dumps(entity, sort_keys=True, indent=4)))
            fk.close()

            outfile = outdir + "entities/" + "img" + "%04d"%(count) + "_search_entry.txt"
            fk = open(outfile,'w')
            fk.write(str(json.dumps(entry, sort_keys=True, indent=4)))
            fk.close()

            if('lt' in entity['binaries']):
                image_url = entity['binaries']['lt']['url']
                outfile = outdir + "lt/" + "img" + "%04d"%(count) + ".jpg"
                save_image(image_url, outfile)

            if('bot_lt' in entity['binaries']):
                image_url = entity['binaries']['bot_lt']['url']
                outfile = outdir + "bot_lt/" + "img" + "%04d"%(count) + ".jpg"
                save_image(image_url,outfile)

            if('bot_smontage' in entity['binaries']):
                image_url = entity['binaries']['bot_smontage']['url']
                outfile = outdir + "bot_smontage/" + "img" + "%04d"%(count) + ".jpg"
                if(save_image(image_url,outfile)):
                    crop_smontage(outfile)

            if('smontage' in entity['binaries']):
                image_url = entity['binaries']['smontage']['url']
                outfile = outdir + "smontage/" + "img" + "%04d"%(count) + ".jpg"
                if(save_image(image_url,outfile)):
                    crop_smontage(outfile)

            count = count + 1
            print "\r" + str(count)

    fm.close()
