import re

import MySQLdb
from lxml import etree

import storage

# Change MySQL credentials here
db = MySQLdb.connect(host="localhost", user="USERNAME", passwd="PASSWORD", db="tdw")
cur = db.cursor()

def getBingClasses():
    query = "SELECT id, name FROM bing_classes"
    cur.execute(query)
    db.commit()

    classes = []
    for idx, name in cur.fetchall():
        classes.append(storage.MyClass(name, idx))

    return classes

def getBingClassID(name):
    query = "SELECT id FROM bing_classes WHERE name='{0}'".format(name)
    cur.execute(query)
    db.commit()

    if cur.rowcount == 0:
        return -1
    else:
        return cur.fetchone()[0]

def getTDWClasses():
    query = "SELECT id, name FROM tdw_classes"
    cur.execute(query)
    db.commit()

    classes = []
    for idx, name in cur.fetchall():
        classes.append(storage.MyClass(name, idx))

    return classes

def getTDWClassID(name):
    query = "SELECT id FROM tdw_classes WHERE name='{0}'".format(name)
    cur.execute(query)
    db.commit()

    if cur.rowcount == 0:
        return -1
    else:
        return cur.fetchone()[0]

def getBingInstances(clID):
    query = "SELECT id, file_id, gtView, filename FROM bing_instances WHERE class_id={0} ORDER BY file_id".format(clID)
    cur.execute(query)
    db.commit()

    instances = []
    for idx, file_idx, gt, filename in cur.fetchall():
        instance = storage.MyInstance(idx, file_idx=file_idx)
        filename = re.sub(r'web_images/(.*?)/', r'web_images/\1/res_400x300/', filename)
        view = storage.MyView(idx, filename, gt)
        instance.views.append(view)
        instances.append(instance)

    return instances

def getTDWInstances(clID):
    query = "SELECT id, gt, file_id, align_offset FROM tdw_instances WHERE class_id={0} AND gt IS NOT NULL ORDER BY file_id".format(clID)
    cur.execute(query)
    db.commit()

    instances = []
    for idx, gt, file_id, align_offset in cur.fetchall():
        instances.append(storage.MyInstance(idx, gt, align_offset, file_id))

    return instances

def getTDWViews(instanceID):
    # get align offset
    query = "SELECT align_offset FROM tdw_instances WHERE id={0}".format(instanceID)
    cur.execute(query)
    db.commit()
    offset = cur.fetchone()[0]

    query = "SELECT id, filename, file_id FROM tdw_views WHERE instance_id={0} ORDER BY file_id".format(instanceID)
    cur.execute(query)
    db.commit()

    views = []
    for idx, filename, file_id in cur.fetchall():
        file_id = int(file_id)
        gt = (file_id - offset) % cur.rowcount
        views.append(storage.MyView(idx, filename, gt))

    return views

def printClasses(classes):
    maxIdxLen = max([len(str(cl.idx)) for cl in classes])
    maxNameLen = max([len(cl.name) for cl in classes])

    print ''.join(['-' for i in range(0, maxIdxLen + maxNameLen + 7)])
    print "| ID{0} | NAME{1} |".format((maxIdxLen - 2) * " ", (maxNameLen - 4) * " ")
    print ''.join(['-' for i in range(0, maxIdxLen + maxNameLen + 7)])

    for cl in classes:
        print "| {0}{1} | {2}{3} |".format(cl.idx, (maxIdxLen - len(str(cl.idx))) * " ", cl.name, (maxNameLen - len(cl.name)) * " ")

    print ''.join(['-' for i in range(0, maxIdxLen + maxNameLen + 7)])

def selectClass(tp):
    print "Select class\n"
    classes = getBingClasses() if tp == 1 else getTDWClasses()
    printClasses(classes)
    choice = ""
    while not (choice in [str(cl.idx) for cl in classes]):
        choice = raw_input("> ")
    return next((cl for cl in classes if str(cl.idx) == choice), None)

def getTagFromXML(xmlFile, tag):
    root = etree.parse(xmlFile)

    return root.find(".//{0}".format(tag))

def replaceTagInXML(xmlFile, tag, data):
    root = etree.parse(xmlFile).getroot()
    tagLoc = root.find(".//{0}".format(tag))
    root.remove(tagLoc)
    root.append(data)

    # write back
    treeString = etree.tostring(root, 'utf-8')
    parser = etree.XMLParser(remove_blank_text=True)
    newlyparsed = etree.fromstring(treeString, parser)
    newText = etree.tostring(newlyparsed, pretty_print=True)
    f = open(xmlFile, 'w')
    f.write(newText)
    f.close()
