from lxml import etree

## Models a single class (one to multiple instances)
class MyClass:
    def __init__(self, name, idx):
        self.name = name
        self.idx = idx
        self.instances = []

    ## Converts this class to an lxml Element and returns it
    def toXML(self):
        root = etree.Element("class")
        root.set("name", self.name)
        root.set("idx", str(self.idx))

        for instance in self.instances:
            subElem = instance.toXML()
            root.append(subElem)

        return root

## Models a single instance (one to multiple views)
class MyInstance:
    def __init__(self, idx, gt=-1, align_offset=0, file_idx=-1):
        self.idx = idx
        self.file_idx = file_idx
        self.gt = gt
        self.align_offset = align_offset
        self.views = []

    ## Converts this instance to an lxml Element and returns it
    def toXML(self):
        root = etree.Element("instance")
        root.set("id", str(self.idx))
        root.set("gt", str(self.gt))
        root.set("align_offset", str(self.align_offset))

        for view in self.views:
            subElem = view.toXML()
            root.append(subElem)

        return root

## Models a single view (single image) of an instance.
class MyView:
    def __init__(self, idx, filename, gt=""):
        self.idx = idx
        self.filename = filename
        self.gt = gt 

    ## Converts this view to an lxml Element and returns it
    def toXML(self):
        root = etree.Element("view")
        root.set("id", str(self.idx))

        filename = etree.SubElement(root, "filename")
        filename.text = self.filename

        if (self.gt != ""):
            gt = etree.SubElement(root, "gt")
            gt.text = str(self.gt)

        return root
