#include "tet/containers/class.h"

namespace tet
{

namespace classification
{

namespace containers
{

size_t Class::size() const
{
    size_t sz = 0;

    for (auto it = _instances.begin(); it != _instances.end(); ++it)
    {
        sz += it->size();
    }

    return sz;
}

tinyxml2::XMLElement* Class::serialize
(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc)
{
    tinyxml2::XMLElement *root = doc->NewElement("class");
    root->SetAttribute("name", _name.c_str());
    parent->LinkEndChild(root);

    for (size_t i = 0; i < _instances.size(); ++i)
    {
        _instances[i].serialize(root, doc);
    }

    return root;

}

void Class::deserialize(const tinyxml2::XMLElement *root)
{
    _name = std::string(root->Attribute("name"));
    for (const tinyxml2::XMLElement *instance = root->FirstChildElement("instance");
         instance != NULL;
         instance = instance->NextSiblingElement("instance"))
    {
        Instance newInstance;
        newInstance.deserialize(instance);
        _instances.push_back(newInstance);
    }
}

std::vector<float> getGTs(const Class & c)
{
    std::vector<float> GTs;

    for (size_t instanceIdx = 0;
         instanceIdx < c.getInstances().size();
         ++instanceIdx)
    {
        const std::vector<float> instanceGTs = getGTs(c.getInstances()[instanceIdx]);

        GTs.insert(GTs.end(), instanceGTs.begin(), instanceGTs.end());
    }

    return GTs;
}

std::vector<float> getScores(const Class & c)
{
    std::vector<float> scores;

    for (size_t instanceIdx = 0;
         instanceIdx < c.getInstances().size();
         ++instanceIdx)
    {
        const std::vector<float> instanceScores = getScores(c.getInstances()[instanceIdx]);

        scores.insert(scores.end(), instanceScores.begin(), instanceScores.end());
    }

    return scores;
}

std::size_t hash_value(const Class & c)
{
    return boost::hash_range(c.getInstances().begin(), c.getInstances().end());
}

} // namespace containers

} // namespace classification

} // namespace tet
