#include "tet/containers/instance.h"

namespace tet
{

namespace classification
{

namespace containers
{

tinyxml2::XMLElement* Instance::serialize
(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc)
{
    tinyxml2::XMLElement *root = doc->NewElement("instance");
    root->SetAttribute("gt", _gt);
    parent->LinkEndChild(root);

    for (size_t i = 0; i < _views.size(); ++i)
    {
        _views[i].serialize(root, doc);
    }

    return root;
}

void Instance::deserialize(const tinyxml2::XMLElement *root)
{
    root->QueryIntAttribute("gt", &_gt);

    for (const tinyxml2::XMLElement *view = root->FirstChildElement("view");
         view != NULL;
         view = view->NextSiblingElement("view"))
    {
        View newView;
        newView.deserialize(view);
        _views.push_back(newView);
    }
}

std::vector<float> getGTs(const Instance & inst)
{
    std::vector<float> GTs(inst.size());

    for (size_t viewIdx = 0;
         viewIdx < inst.getViews().size();
         ++viewIdx)
    {
        GTs[viewIdx] = inst.getViews()[viewIdx].getGT();
    }

    return GTs;
}

std::vector<float> getScores(const Instance & inst)
{
    std::vector<float> scores(inst.size());

    for (size_t viewIdx = 0;
         viewIdx < inst.getViews().size();
         ++viewIdx)
    {
        scores[viewIdx] = inst.getViews()[viewIdx].getScore();
    }

    return scores;
}


std::size_t hash_value(const Instance & instance)
{
    return boost::hash_range(instance.getViews().begin(), instance.getViews().end());
}

} // namespace containers

} // namespace classification

} // namespace tet
