#include "tet/containers/set.h"

#include "tet/eval/evaldata.h"
#include "tet/eval/evaldatapoint.h"

namespace tet
{

namespace classification
{

namespace containers
{

size_t Set::size() const
{
    size_t sz = 0;

    for (auto it = _classes.begin(); it != _classes.end(); ++it)
    {
        sz += it->size();
    }
    
    return sz;
}

tinyxml2::XMLElement* Set::serialize
(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc)
{
    tinyxml2::XMLElement *root = doc->NewElement("set");
    parent->LinkEndChild(root);

    for (size_t i = 0; i < _classes.size(); ++i)
    {
        _classes[i].serialize(root, doc);
    }

    return root;
}

void Set::deserialize(const tinyxml2::XMLElement *root)
{
    for (const tinyxml2::XMLElement *classTag = root->FirstChildElement("class");
         classTag != NULL;
         classTag = classTag->NextSiblingElement("class"))
    {
        Class newClass;
        newClass.deserialize(classTag);
        _classes.push_back(newClass);
    }
}

std::vector<float> getGTs(const Set & set)
{
    std::vector<float> GTs;

    for (size_t classIdx = 0;
         classIdx < set.getClasses().size();
         ++classIdx)
    {
        const std::vector<float> classGTs = getGTs(set.getClasses()[classIdx]);

        GTs.insert(GTs.end(), classGTs.begin(), classGTs.end());
    }

    return GTs;
}

std::vector<float> getScores(const Set & set)
{
    std::vector<float> scores;

    for (size_t classIdx = 0;
         classIdx < set.getClasses().size();
         ++classIdx)
    {
        const std::vector<float> classScores = getScores(set.getClasses()[classIdx]);

        scores.insert(scores.end(), classScores.begin(), classScores.end());
    }

    return scores;
}

eval::EvalData getEvalData(const Set & set)
{
    std::vector<float> gts = getGTs(set);
    std::vector<float> scores = getScores(set);

    eval::EvalData evalData;
    for (unsigned int i = 0; i < gts.size(); ++i)
    {
        eval::EvalDataPoint dp(scores[i], gts[i]);
        evalData.addData(dp);
    }

    return evalData;

}

std::size_t hash_value(const Set & set)
{
    return boost::hash_range(set.getClasses().begin(), set.getClasses().end());
}

} // namespace containers

} // namespace classification

} // namespace tet
