#include <iostream>

#include "tet/io/sampleloader.h"
#include "tet/io/eigenio.h"

#include "tet/util/md5.h"

#include <sys/stat.h>

#include "tet/containers/view.h"

namespace tet
{

namespace classification
{

namespace containers
{

tinyxml2::XMLElement* View::serialize
(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc)
{
    tinyxml2::XMLElement *root = doc->NewElement("view");
    parent->LinkEndChild(root);

    tinyxml2::XMLElement *filename = doc->NewElement("filename");
    filename->SetText(_filename.c_str());
    root->LinkEndChild(filename);

    if (_scoreComputed)
    {
        tinyxml2::XMLElement *score = doc->NewElement("score");
        score->SetText(_score);
        root->LinkEndChild(score);
    }

    if (_gtSet)
    {
        tinyxml2::XMLElement *gt = doc->NewElement("gt");
        gt->SetText(_gt);
        root->LinkEndChild(gt);
    }

    return root;
}

void View::deserialize(const tinyxml2::XMLElement *root)
{
    if (const tinyxml2::XMLElement *filename = root->FirstChildElement("filename"))
    {
        setFilename(std::string(filename->GetText()));
    }

    if (const tinyxml2::XMLElement *score = root->FirstChildElement("score"))
    {
        _scoreComputed = true;
        score->QueryFloatText(&_score);
    }

    if (const tinyxml2::XMLElement *gt = root->FirstChildElement("gt"))
    {
        _gtSet = true;
        gt->QueryFloatText(&_gt);
    }
    
}

void View::_computeMD5()
{
    _md5 = tet::classification::util::computeMD5(_filename);
}

std::size_t hash_value(const View & view)
{
    return boost::hash<std::string>()(view.getMD5());
}

} // namespace containers

} // namespace classification

} // namespace tet
