#include <vector>

#include "tet/descriptors/siftdescriptor.h"

namespace tet
{

namespace classification
{

namespace descriptors
{

Descriptor<_internal::SIFTDescriptorTraits>::VisType SIFTDescriptor::getVisualization(const FeatureType &features) const
{
    return cv::Mat();
}

Descriptor<_internal::SIFTDescriptorTraits>::FeatureType SIFTDescriptor::getDescriptors(const InputType &img, VisType* vis) const
{
    const float *imgPtr;
    cv::Mat concat;
    if (img.channels() > 1)
    {
        std::vector<cv::Mat> channels;
        for (int i = 0; i < img.channels(); ++i)
        {
            channels.push_back(cv::Mat(img.rows, img.cols, CV_32FC1));
        }
        cv::split(img, channels);

        cv::vconcat(channels, concat);

        imgPtr = concat.ptr<float>(0);
    } else {
        imgPtr = img.ptr<float>(0);
    }


    VlDsiftFilter * filter = vl_dsift_new_basic(img.cols, img.rows, _step, _binSize);
    vl_dsift_process(filter, imgPtr);
    std::cout << vl_dsift_get_keypoint_num(filter) << std::endl;

    float * descriptors = const_cast<float*>(vl_dsift_get_descriptors(filter));
    Eigen::MatrixXf descriptorsEig(vl_dsift_get_keypoint_num(filter), vl_dsift_get_descriptor_size(filter));

    descriptorsEig = Eigen::Map<Eigen::MatrixXf>(descriptors, vl_dsift_get_keypoint_num(filter), vl_dsift_get_descriptor_size(filter));

    vl_dsift_delete(filter);

    return descriptorsEig;
}

tinyxml2::XMLElement* SIFTDescriptor::serialize(tinyxml2::XMLElement* parent,
                                                tinyxml2::XMLDocument* doc)
{
    tinyxml2::XMLElement *root = doc->NewElement("descriptor");
    root->SetAttribute("type", "sift");
    parent->LinkEndChild(root);
    
    tinyxml2::XMLElement *binSize = doc->NewElement("binSize");
    binSize->SetText(_binSize);
    root->LinkEndChild(binSize);

    tinyxml2::XMLElement *step = doc->NewElement("step");
    step->SetText(_step);
    root->LinkEndChild(step);

    return root;

}

void SIFTDescriptor::deserialize(const tinyxml2::XMLElement* root)
{
    if (const tinyxml2::XMLElement *binSize = root->FirstChildElement("binSize"))
    {
        binSize->QueryIntText(&_binSize);
    }

    if (const tinyxml2::XMLElement *step = root->FirstChildElement("step"))
    {
        step->QueryIntText(&_step);
    }

}

} // namespace descriptors

} // namespace classification

} // namespace tet
