#include <limits>
#include <fstream>

#include "boost/lexical_cast.hpp"

#include "eigen3/Eigen/Core"
#include "eigen3/Eigen/Eigenvalues"
#include "opencv2/core/eigen.hpp"

#include "tet/encodings/histencoding.h"

#include "tet/util/stringutils.h"


namespace tet
{

namespace classification
{

namespace encodings
{

/*void HistEncoding::init(InitStruct initStruct)
{
    if (_pcaDim > 0)
    {
        SampleType centered = initStruct.samples->rowwise() - initStruct.samples->colwise().mean();
        SampleType cov = centered.adjoint() * centered;
        Eigen::SelfAdjointEigenSolver<SampleType> eig(cov);

        _pcaBasis = eig.eigenvectors().rightCols(_pcaDim);
        std::cout << _pcaBasis.rows() << ", " << _pcaBasis.cols() << std::endl;
        _usePCA = true;

        *(initStruct.samples) = *(initStruct.samples) * _pcaBasis;
    }

    cv::Mat samplesCV, labels, centers;

    // convert samples to OpenCV format
    cv::eigen2cv(*(initStruct.samples), samplesCV);
    samplesCV.convertTo(samplesCV, CV_32F);

    // compute kmeans
    kmeans(samplesCV, _k, labels, cv::TermCriteria(cv::TermCriteria::EPS + cv::TermCriteria::COUNT, 10, 0.001),
           3, cv::KMEANS_PP_CENTERS, centers);

    // convert back to Eigen
    cv2eigen(centers, _centers);

    _tree = new tet::classification::util::ANNTree<Scalar>(_centers);

    _computed = true;

}*/

HistEncoding::FeatureType HistEncoding::computeFeatures(SampleType samples) const
{
    if (_usePCA)
    {
        samples = samples * _pcaBasis;
    }
    
    FeatureType hist(_k, 1);
    hist.setZero();

    for (int i = 0; i < samples.rows(); ++i)
    {
        hist(_tree->findNN(samples.row(i)), 0) += 1;
    }

    if (samples.rows() > 0)
    {
        hist = hist.normalized();
    }

    return hist;
}

void HistEncoding::saveCenters()
{
    std::ofstream out;
    out.open(".centers.txt");

    for (unsigned int i = 0; i < _centers.rows(); ++i)
    {
        for (unsigned int j = 0; j < _centers.row(i).cols() - 1; ++j)
        {
            out << _centers.row(i)(j) << ", ";
        }
        out << _centers.row(i)(_centers.row(i).cols() - 1) << std::endl;
    }

    out.close();
}

void HistEncoding::printCenter(size_t idx)
{
    for (unsigned int i = 0; i < _centers.row(idx).cols() - 1; ++i)
    {
        std::cout << _centers.row(idx)(i) << ", ";
    }
    std::cout << _centers.row(idx)(_centers.row(idx).cols() - 1) << std::endl;
}


tinyxml2::XMLElement* HistEncoding::serializeSpec(tinyxml2::XMLElement* parent, tinyxml2::XMLDocument* doc)
{
    tinyxml2::XMLElement *k = doc->NewElement("k");
    k->SetText(_k);
    parent->LinkEndChild(k);

    if (_usePCA)
    {
        tinyxml2::XMLElement *pca = doc->NewElement("pca");
        pca->SetAttribute("dim", _pcaDim);
        parent->LinkEndChild(pca);
    }

    if (_computed)
    {
        tinyxml2::XMLElement *centers = doc->NewElement("centers");
        parent->LinkEndChild(centers);

        for (unsigned int i = 0; i < _centers.rows(); ++i)
        {
            tinyxml2::XMLElement *center = doc->NewElement("center");
            centers->LinkEndChild(center);

            std::stringstream row;
            for (unsigned int j = 0; j < _centers.row(i).cols() - 1; ++j)
            {
                row << _centers.row(i)(j) << " ";
            }
            row << _centers.row(i)(_centers.row(i).cols() - 1);
            center->SetText(row.str().c_str());
        }
    }

    return parent;
}

void HistEncoding::deserializeSpec(const tinyxml2::XMLElement* root)
{
    if (const tinyxml2::XMLElement *k = root->FirstChildElement("k"))
    {
        k->QueryIntText(&_k);
    }

    if (const tinyxml2::XMLElement *pca = root->FirstChildElement("pca"))
    {
        _usePCA = true;
        pca->QueryIntText(&_pcaDim);
    }

    if (const tinyxml2::XMLElement *centers = root->FirstChildElement("centers"))
    {
        _computed = true;

        const tinyxml2::XMLElement *center = centers->FirstChildElement("center");
        std::vector<std::string> elems = stringSplit(std::string(center->GetText()), ' ');
        _centers = SampleType(_k, elems.size());
        std::cout << elems.size() << std::endl;
        int row = 0;
        for (size_t i = 0; i < elems.size(); ++i)
        {
            _centers(row, i) = boost::lexical_cast<Scalar>(elems[i]);
        }

        row = 1;
        for (center = center->NextSiblingElement("center");
             center != NULL;
             center = center->NextSiblingElement("center"))
        {
            elems = stringSplit(std::string(center->GetText()), ' ');
            for (unsigned int i = 0; i < _centers.cols(); ++i)
            {
                _centers(row, i) = boost::lexical_cast<Scalar>(elems[i]);
            }
            row += 1;
        }

        _tree = new tet::classification::util::ANNTree<Scalar>(_centers);
    }
}

} // namespace encodings

} // namespace classification

} // namespace tet
