#include <algorithm>
#include <iostream>
#include <cmath>

#include <boost/lexical_cast.hpp>

#include "tet/eval/madmetric.h"

namespace tet
{

namespace classification
{

namespace eval
{

void MADMetric::compute(const EvalData &data)
{
    ResultType accum = 0;
    
    std::for_each(data.begin(), data.end(), [&] (EvalDataPoint x)
        { accum.value += std::abs(x.getGT() - x.getComputed()); }
    );

    _result.setValue(accum.value / data.size());
    _computed = true;

    std::cout << "MAD Metric: " << _result.value << std::endl;
}

EvalResult::Ptr MADMetric::getResult() const
{
    if (_computed)
    {
        return std::make_shared<ResultType>(_result);
    } else {
        return std::shared_ptr<ResultType>();
    }
}

void MADMetric::deserialize
(const tinyxml2::XMLElement* root)
{
    const char * text = root->GetText();
    if (text)
    {
        _computed = true;
        _result.value = boost::lexical_cast<float>(text);
    }

}

tinyxml2::XMLElement* MADMetric::serialize
(tinyxml2::XMLElement *parent, tinyxml2::XMLDocument* doc)
{
    tinyxml2::XMLElement* root = doc->NewElement("eval");
    root->SetAttribute("type", "mad");
    root->SetText(_result.value);
    parent->LinkEndChild(root);

    return parent;
}

} // namespace eval

} // namespace classification

} // namespace tet
