#include <iostream>
#include <fstream>

#include "tet/descriptors/hogdescriptor.h"
#include "tet/encodings/kcencoding.h"
#include "tet/encodings/histencoding.h"

#include "tet/containers/set.h"
#include "tet/app/classifierapp.h"

#include "boost/lexical_cast.hpp"


int main(int argc, char* argv[])
{
    if (argc < 3)
    {
        std::cout << "Usage: " << argv[0] << " [settings] [outprefix]" << std::endl;
        return -1;
    }

    typedef tet::classification::descriptors::HOGDescriptor DescriptorType;
    typedef tet::classification::encodings::KCEncoding EncoderType;

    tet::classification::app::ClassifierApp<DescriptorType, EncoderType> app;
    app.readFile(argv[1]);
    app.setUseCached(true);
    app.init();

    std::string outprefix = std::string(argv[2]);

    int numDim = app.getSVM().getPCA().getEigenvectors().cols();

    std::vector<EncoderType::FeatureType*> encodings = app.getCustom()->getAllEncodings();
    std::vector<std::string> files = app.getCustom()->getAllFilenames();

    for (int dim = 0; dim < numDim; ++dim)
    {

        float minDist = 1e8;
        std::stringstream outfile;
        outfile << outprefix << dim << ".txt";

        std::vector<std::pair<float, std::string> > scores;

        for (size_t i = 0; i < encodings.size(); ++i)
        {
            float score = (*(encodings[i])).transpose() * app.getSVM().getPCA().getEigenvectors().col(numDim - (dim + 1));
            std::string filename = files[i];
            scores.push_back(std::pair<float, std::string>(score, filename));
        }

        std::sort(scores.begin(), scores.end());

        std::ofstream out;
        out.open(outfile.str());

        for (size_t i = 0; i < scores.size(); ++i)
        {
            out << scores[i].first << ": " << scores[i].second << std::endl;
        }

        out.close();
    }

}
