#include <iostream>

#include <vector>

#include "tet/descriptors/hogdescriptor.h"
#include "tet/encodings/kcencoding.h"
#include "tet/encodings/histencoding.h"

#include "tet/containers/set.h"
#include "tet/app/classifierapp.h"

int main(int argc, char* argv[])
{
    if (argc < 3)
    {
        std::cout << "Usage: " << argv[0] << " [settings] [distributionTraining] [distributionTest]" << std::endl;
        return -1;
    }

    typedef tet::classification::descriptors::HOGDescriptor DescriptorType;
    typedef tet::classification::encodings::HistEncoding EncoderType;

    tet::classification::app::ClassifierApp<DescriptorType, EncoderType> app;
    app.readFile(argv[1]);
    app.init();

    std::vector<EncoderType::FeatureType*> posEncodings = app.getPositives()->getAllEncodings();
    //std::vector<EncoderType::FeatureType*> negEncodings = app.getNegatives()->getAllEncodings();
    std::vector<EncoderType::FeatureType*> testEncodings = app.getTests()->getAllEncodings();

    EncoderType::FeatureType trainAvg(posEncodings[0]->rows(), 1);
    trainAvg.setZero();
    int N = 0;
    for (size_t i = 0; i < posEncodings.size(); ++i)
    {
        trainAvg += (*(posEncodings[i]));
        N += 1;
    }

    trainAvg /= N;

    EncoderType::FeatureType testAvg(testEncodings[0]->rows(), 1);
    testAvg.setZero();
    N = 0;
    for (size_t i = 0; i < testEncodings.size(); ++i)
    {
        testAvg += (*(testEncodings[i]));
        N += 1;
    }

    testAvg /= N;

    std::ofstream trainFile;
    trainFile.open(argv[2]);
    for (size_t i = 0; i < trainAvg.rows(); ++i)
    {
        trainFile << trainAvg(i) << std::endl;
    }
    trainFile.close();

    std::ofstream testFile;
    testFile.open(argv[3]);
    for (size_t i = 0; i < testAvg.rows(); ++i)
    {
        testFile << testAvg(i) << std::endl;
    }
    testFile.close();


}
