#include <iostream>
#include <fstream>

#include <vector>

#include "tet/descriptors/hogdescriptor.h"
#include "tet/encodings/kcencoding.h"

#include "tet/containers/set.h"
#include "tet/app/regressorapp.h"
#include "tet/app/runsettings.h"
#include "tet/app/appfactory.h"
#include "tet/app/appbase.h"

#include "tet/processing/dataprocessing.h"

#include "tet/io/eigenio.h"

#define CACHE_DIR "/home/moos/phd/proj/search/data/cache"

int main(int argc, char* argv[])
{
    if (argc < 3)
    {
        std::cout << "Usage: " << argv[0] << " [settings] [featureFilePrefix]" << std::endl;
        return -1;
    }

    typedef tet::classification::descriptors::HOGDescriptor DescriptorType;
    typedef tet::classification::encodings::KCEncoding EncoderType;
    typedef tet::classification::app::RegressorApp<DescriptorType, EncoderType> AppType;

    tet::classification::app::RunSettings settings;

    tet::classification::app::AppFactory appFactory;
    AppType * app = dynamic_cast<AppType*>(appFactory.createApp("reg", "hog", "kc"));

    app->readFile(argv[1]);

    tet::classification::processing::GetSetEncodingsCached<DescriptorType, EncoderType> getSetEncodings(CACHE_DIR);

    std::cout << "Getting all encodings..." << std::endl;
    std::vector<EncoderType::FeatureType*> posEncodings = 
        getSetEncodings(app->getSamples(), app->getDescriptor(), app->getEncoder());

    std::cout << "Writing encodings to files..." << std::endl;
    
    app->writeFile(argv[1]);

    std::ofstream out;
    out.open(argv[2]);
    for (size_t i = 0; i < posEncodings.size(); ++i)
    {
        out << (*(posEncodings[i]))(0);
        for (size_t j = 1; j < posEncodings[i]->rows(); ++j)
        {
            out << ", " << (*(posEncodings[i]))(j);
        }
        out << std::endl;
    }

    out.close();
    
}
