#include <iostream>
#include <fstream>

#include <vector>

#include "tet/descriptors/hogdescriptor.h"
#include "tet/encodings/kcencoding.h"
#include "tet/encodings/histencoding.h"

#include "tet/containers/set.h"
#include "tet/app/classifierapp.h"

#include "tet/io/eigenio.h"

int main(int argc, char* argv[])
{
    if (argc < 4)
    {
        std::cout << "Usage: " << argv[0] << " [imfile [settings] [outfile]" << std::endl;
        return -1;
    }

    typedef tet::classification::descriptors::HOGDescriptor DescriptorType;
    typedef tet::classification::encodings::KCEncoding EncoderType;

    tet::classification::app::ClassifierApp<DescriptorType, EncoderType> app;
    app.readFile(argv[2]);
    //app.init();

    tet::classification::containers::View<DescriptorType, EncoderType> view;
    view.setDescriptor(app.getDescriptor());
    view.setFilename(std::string(argv[1]));

    std::cout << "Getting encodings..." << std::endl;
    EncoderType::FeatureType encoding = app.getEncoding()->computeFeatures(app.getDescriptor()->getDescriptors(*view.getInput()));

    std::ofstream outFile;
    outFile.open(argv[3]);
    outFile << encoding;

    outFile.close();

}
